@@ -11,6 +11,8 @@ https://bugs.launchpad.net/ubuntu/+source/cloud-init/+bug/1886531
 """
 import pytest
 
+from tests.integration_tests.util import verify_clean_log
+
 
 USER_DATA = """\
 #cloud-config
@@ -24,4 +26,4 @@ class TestLp1886531:
     @pytest.mark.user_data(USER_DATA)
     def test_lp1886531(self, client):
         log_content = client.read_from_file("/var/log/cloud-init.log")
-        assert "WARNING" not in log_content
+        verify_clean_log(log_content)
