@@ -1,39 +1,23 @@
-            #    Copyright (C) 2013 Hewlett-Packard Development Company, L.P.
+# This file is part of cloud-init. See LICENSE file for license information.
+
+#    Copyright (C) 2013 Hewlett-Packard Development Company, L.P.
 #
 #    Author: Juerg Haefliger <juerg.haefliger@hp.com>
 #
 #    Based on test_handler_set_hostname.py
 #
-#    This program is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License version 3, as
-#    published by the Free Software Foundation.
-#
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-from cloudinit.config import cc_seed_random
-
-import base64
+#    This file is part of cloud-init. See LICENSE file for license information.
 import gzip
-import tempfile
-
-from StringIO import StringIO
-
-from cloudinit import cloud
-from cloudinit import distros
-from cloudinit import helpers
-from cloudinit import util
-
-from cloudinit.sources import DataSourceNone
-
-from tests.unittests import helpers as t_help
-
 import logging
+import tempfile
+from io import BytesIO
+
+from cloudinit import subp
+from cloudinit import util
+from cloudinit.config import cc_seed_random
+from cloudinit.tests import helpers as t_help
+
+from tests.unittests.util import get_cloud
 
 LOG = logging.getLogger(__name__)
 
@@ -45,8 +29,8 @@ class TestRandomSeed(t_help.TestCase):
         self.unapply = []
 
         # by default 'which' has nothing in its path
-        self.apply_patches([(util, 'which', self._which)])
-        self.apply_patches([(util, 'subp', self._subp)])
+        self.apply_patches([(subp, 'which', self._which)])
+        self.apply_patches([(subp, 'subp', self._subp)])
         self.subp_called = []
         self.whichdata = {}
 
@@ -69,21 +53,12 @@ class TestRandomSeed(t_help.TestCase):
         return
 
     def _compress(self, text):
-        contents = StringIO()
+        contents = BytesIO()
         gz_fh = gzip.GzipFile(mode='wb', fileobj=contents)
         gz_fh.write(text)
         gz_fh.close()
         return contents.getvalue()
 
-    def _get_cloud(self, distro, metadata=None):
-        paths = helpers.Paths({})
-        cls = distros.fetch(distro)
-        ubuntu_distro = cls(distro, {}, paths)
-        ds = DataSourceNone.DataSourceNone({}, ubuntu_distro, paths)
-        if metadata:
-            ds.metadata = metadata
-        return cloud.Cloud(ds, paths, {}, ubuntu_distro, None)
-
     def test_append_random(self):
         cfg = {
             'random_seed': {
@@ -91,12 +66,12 @@ class TestRandomSeed(t_help.TestCase):
                 'data': 'tiny-tim-was-here',
             }
         }
-        cc_seed_random.handle('test', cfg, self._get_cloud('ubuntu'), LOG, [])
+        cc_seed_random.handle('test', cfg, get_cloud('ubuntu'), LOG, [])
         contents = util.load_file(self._seed_file)
-        self.assertEquals("tiny-tim-was-here", contents)
+        self.assertEqual("tiny-tim-was-here", contents)
 
     def test_append_random_unknown_encoding(self):
-        data = self._compress("tiny-toe")
+        data = self._compress(b"tiny-toe")
         cfg = {
             'random_seed': {
                 'file': self._seed_file,
@@ -105,10 +80,10 @@ class TestRandomSeed(t_help.TestCase):
             }
         }
         self.assertRaises(IOError, cc_seed_random.handle, 'test', cfg,
-                          self._get_cloud('ubuntu'), LOG, [])
+                          get_cloud('ubuntu'), LOG, [])
 
     def test_append_random_gzip(self):
-        data = self._compress("tiny-toe")
+        data = self._compress(b"tiny-toe")
         cfg = {
             'random_seed': {
                 'file': self._seed_file,
@@ -116,12 +91,12 @@ class TestRandomSeed(t_help.TestCase):
                 'encoding': 'gzip',
             }
         }
-        cc_seed_random.handle('test', cfg, self._get_cloud('ubuntu'), LOG, [])
+        cc_seed_random.handle('test', cfg, get_cloud('ubuntu'), LOG, [])
         contents = util.load_file(self._seed_file)
-        self.assertEquals("tiny-toe", contents)
+        self.assertEqual("tiny-toe", contents)
 
     def test_append_random_gz(self):
-        data = self._compress("big-toe")
+        data = self._compress(b"big-toe")
         cfg = {
             'random_seed': {
                 'file': self._seed_file,
@@ -129,12 +104,12 @@ class TestRandomSeed(t_help.TestCase):
                 'encoding': 'gz',
             }
         }
-        cc_seed_random.handle('test', cfg, self._get_cloud('ubuntu'), LOG, [])
+        cc_seed_random.handle('test', cfg, get_cloud('ubuntu'), LOG, [])
         contents = util.load_file(self._seed_file)
-        self.assertEquals("big-toe", contents)
+        self.assertEqual("big-toe", contents)
 
     def test_append_random_base64(self):
-        data = base64.b64encode('bubbles')
+        data = util.b64e('bubbles')
         cfg = {
             'random_seed': {
                 'file': self._seed_file,
@@ -142,12 +117,12 @@ class TestRandomSeed(t_help.TestCase):
                 'encoding': 'base64',
             }
         }
-        cc_seed_random.handle('test', cfg, self._get_cloud('ubuntu'), LOG, [])
+        cc_seed_random.handle('test', cfg, get_cloud('ubuntu'), LOG, [])
         contents = util.load_file(self._seed_file)
-        self.assertEquals("bubbles", contents)
+        self.assertEqual("bubbles", contents)
 
     def test_append_random_b64(self):
-        data = base64.b64encode('kit-kat')
+        data = util.b64e('kit-kat')
         cfg = {
             'random_seed': {
                 'file': self._seed_file,
@@ -155,9 +130,9 @@ class TestRandomSeed(t_help.TestCase):
                 'encoding': 'b64',
             }
         }
-        cc_seed_random.handle('test', cfg, self._get_cloud('ubuntu'), LOG, [])
+        cc_seed_random.handle('test', cfg, get_cloud('ubuntu'), LOG, [])
         contents = util.load_file(self._seed_file)
-        self.assertEquals("kit-kat", contents)
+        self.assertEqual("kit-kat", contents)
 
     def test_append_random_metadata(self):
         cfg = {
@@ -166,35 +141,38 @@ class TestRandomSeed(t_help.TestCase):
                 'data': 'tiny-tim-was-here',
             }
         }
-        c = self._get_cloud('ubuntu', {'random_seed': '-so-was-josh'})
+        c = get_cloud('ubuntu', metadata={'random_seed': '-so-was-josh'})
         cc_seed_random.handle('test', cfg, c, LOG, [])
         contents = util.load_file(self._seed_file)
-        self.assertEquals('tiny-tim-was-here-so-was-josh', contents)
+        self.assertEqual('tiny-tim-was-here-so-was-josh', contents)
 
-    def test_seed_command_not_provided_pollinate_available(self):
-        c = self._get_cloud('ubuntu', {})
+    def test_seed_command_provided_and_available(self):
+        c = get_cloud('ubuntu')
         self.whichdata = {'pollinate': '/usr/bin/pollinate'}
-        cc_seed_random.handle('test', {}, c, LOG, [])
+        cfg = {'random_seed': {'command': ['pollinate', '-q']}}
+        cc_seed_random.handle('test', cfg, c, LOG, [])
 
         subp_args = [f['args'] for f in self.subp_called]
         self.assertIn(['pollinate', '-q'], subp_args)
 
-    def test_seed_command_not_provided_pollinate_not_available(self):
-        c = self._get_cloud('ubuntu', {})
+    def test_seed_command_not_provided(self):
+        c = get_cloud('ubuntu')
         self.whichdata = {}
         cc_seed_random.handle('test', {}, c, LOG, [])
 
         # subp should not have been called as which would say not available
-        self.assertEquals(self.subp_called, list())
+        self.assertFalse(self.subp_called)
 
     def test_unavailable_seed_command_and_required_raises_error(self):
-        c = self._get_cloud('ubuntu', {})
+        c = get_cloud('ubuntu')
         self.whichdata = {}
+        cfg = {'random_seed': {'command': ['THIS_NO_COMMAND'],
+                               'command_required': True}}
         self.assertRaises(ValueError, cc_seed_random.handle,
-            'test', {'random_seed': {'command_required': True}}, c, LOG, [])
+                          'test', cfg, c, LOG, [])
 
     def test_seed_command_and_required(self):
-        c = self._get_cloud('ubuntu', {})
+        c = get_cloud('ubuntu')
         self.whichdata = {'foo': 'foo'}
         cfg = {'random_seed': {'command_required': True, 'command': ['foo']}}
         cc_seed_random.handle('test', cfg, c, LOG, [])
@@ -202,7 +180,7 @@ class TestRandomSeed(t_help.TestCase):
         self.assertIn(['foo'], [f['args'] for f in self.subp_called])
 
     def test_file_in_environment_for_command(self):
-        c = self._get_cloud('ubuntu', {})
+        c = get_cloud('ubuntu')
         self.whichdata = {'foo': 'foo'}
         cfg = {'random_seed': {'command_required': True, 'command': ['foo'],
                                'file': self._seed_file}}
@@ -223,3 +201,5 @@ def apply_patches(patches):
         setattr(ref, name, replace)
         ret.append((ref, name, orig))
     return ret
+
+# vi: ts=4 expandtab
